require 'spec_helper'

describe GalleriesController do

  describe "GET index" do
    before do
      @tenant = get_tenant
      @tenant.galleries.delete_all
      @gallery_1 = Factory(:gallery, :name => 'gallery_1', :tenant => @tenant)
      @gallery_2 = Factory(:gallery, :name => 'gallery_2', :tenant => @tenant)
      Factory(:gallery, :name => 'gallery_3', :tenant => get_admin)
    end

    def do_request
      get :index, :tenant_id => @tenant.to_param
    end

    it "should return all galleries belong to this tenant" do
      do_request
      response.should be_success
      assigns(:title).should == "Galleries Index Page"
      assigns(:galleries).should =~ [@gallery_1, @gallery_2]
    end
  end

  describe "GET show" do
    before do
      @gallery = get_landscape_gallery
      4.times do
        Factory(:album, :gallery => @gallery, :tenant => @gallery.tenant)
      end
    end
    def do_request
      get :show, :tenant_id => get_landscape_gallery.tenant.to_param, :id => get_landscape_gallery.to_param
    end

    it "should return the correct tenant" do
      do_request
      response.should be_success
      assigns(:title).should == "Galleries Show Page"
      assigns(:gallery).should == get_landscape_gallery
      assigns(:albums).to_a.size.should == 5
      response.should have_selector("#galleries_show li.album", :count => 5)
    end
  end

  describe "GET new" do
    before do
      @admin = get_admin
      @tenant = get_tenant
      @another_tenant = Factory(:tenant, :tenant_name => 'another_tenant', :email => "another_tenant@gmail.com")
    end
    def do_request
      get :new, :tenant_id => get_tenant
    end

    context 'guest access' do
      it "should raise access denied" do
        lambda {do_request}.should raise_error(CanCan::AccessDenied)
        assigns(:gallery).should_not be_present
      end
    end

    context 'user access' do
      context 'owner access' do
        before do
          sign_in @tenant
        end

        it "should be success" do
          do_request
          response.should be_success
          assigns(:gallery).should be_present
        end
      end

      context 'another tenant access' do
        before do
          sign_in @another_tenant
        end
        it "should not raise error" do
          lambda {do_request}.should raise_error(CanCan::AccessDenied)
          assigns(:gallery).should_not be_present
        end
      end

      context 'admin access' do
        before do
          sign_in @admin
        end

        it "should be success" do
          do_request
          response.should be_success
          assigns(:gallery).should be_present
        end
      end
    end
  end

  describe "GET create" do
    before do
      sign_in get_tenant
      @new_gallery = Factory.attributes_for(:portrait_gallery).except(:name).merge(:name => 'new_gallery')
    end

    def do_request
      post :create, :tenant_id => get_tenant, :gallery => @new_gallery
    end

    it "should create a new gallery" do
      expect{
        do_request
      }.should change(Gallery, :count).by(1)
    end

    it "should redirect to show page of the new gallery" do
      do_request
      response.should redirect_to(tenant_gallery_path(get_tenant,assigns(:gallery).to_param))
    end
  end

  describe "GET edit" do
    before do
      @gallery = get_landscape_gallery
      sign_in @gallery.tenant
    end

    def do_request
      get :edit, :tenant_id => @gallery.tenant.to_param, :id => @gallery.to_param
    end

    it "should be successful" do
      do_request
      response.should be_success
      assigns(:gallery).should == @gallery
    end
  end

  describe "PUT update" do
    before do
      @gallery = Factory(:portrait_gallery)
      @new_gallery_attrs = @gallery.attributes.merge(:name => 'new_portrait_gallery')
      sign_in @gallery.tenant
    end

    def do_request
      put :update, :tenant_id => @gallery.tenant.to_param, :id => @gallery.id, :gallery => @new_gallery_attrs
    end

    it "should be successful" do
      do_request
      @gallery.reload
      response.should redirect_to(tenant_gallery_path(@gallery.tenant, @gallery))
    end

    it "should update the gallery name" do
      do_request
      @gallery.reload
      @gallery.name.should == "new_portrait_gallery"
    end
  end

  describe "DELETE destroy" do
    before do
      @gallery = Factory(:portrait_gallery)
      sign_in @gallery.tenant
    end

    def do_request
      delete :destroy, :tenant_id => @gallery.tenant.to_param, :id => @gallery.to_param
    end

    it "should destroy the gallery" do
      expect{
        do_request
      }.should change(Gallery, :count).by(-1)
    end

    it "should be redirected to gallery index page" do
      do_request
      response.should redirect_to tenant_galleries_path
    end
  end

end
